import pandas as pd
import os
import re
import logging
import PyLibrary.funciones_comunes as fc
from translation_config import TranslationConfig

# Definir el logger a nivel de módulo
logger = None


def preprocesar_importacion(
    codigo_tipo_PLC, codigo_columna_maestra, df_importacion, logger
):
    filas_validas = []

    progress_bar = fc.ProgressBar(
        len(df_importacion), prefix="Preprocesando filas:", suffix="Completado"
    )

    for index, fila in df_importacion.iterrows():
        clave_original = fila[codigo_columna_maestra]

        # Verificar si la celda de la columna maestra está vacía o solo contiene espacios en blanco
        if pd.isna(clave_original) or str(clave_original).strip() == "":
            logger.info(
                f"Fila {index}: Ignorada - Celda de columna maestra vacía o solo con espacios en blanco"
            )
            progress_bar.increment()
            continue

        nueva_fila = fila.copy()

        clave_sustituida = fc.compactar_celda_clave(
            codigo_tipo_PLC, str(nueva_fila[codigo_columna_maestra])
        )
        nueva_fila[codigo_columna_maestra] = clave_sustituida

        for columna in df_importacion.columns:
            if columna != codigo_columna_maestra and fc.es_columna_tipo_xxYY(columna):
                nueva_fila[columna] = fc.compactar_celda_traducida(
                    codigo_tipo_PLC, nueva_fila[columna]
                )

        filas_validas.append(nueva_fila)

        logger.info(f"Fila {index}: Aceptada - Clave '{clave_sustituida}' procesada")
        progress_bar.increment()

    progress_bar.finish()
    df_valido = pd.DataFrame(filas_validas, columns=df_importacion.columns)
    return df_valido


def importar(config: TranslationConfig, archivo_importacion):
    logger.info("    ....    ")
    logger.info(f"Iniciando importacion en {config.master_name}")

    archivo_maestro = os.path.join(config.work_dir, config.master_name)

    if not archivo_importacion:
        logger.info("No se seleccionó ningún archivo para importar.")
        return

    if not os.path.exists(archivo_maestro):
        df_maestro = pd.DataFrame(columns=[config.codigo_columna_maestra])
    else:
        df_maestro = fc.read_dataframe_with_cleanup_retries(archivo_maestro)

    df_importacion = fc.read_dataframe_with_cleanup_retries(archivo_importacion)

    df_importacion = preprocesar_importacion(
        config.codigo_tipo_PLC, config.codigo_columna_maestra, df_importacion, logger
    )

    claves_maestro = set(df_maestro[config.codigo_columna_maestra].dropna().astype(str))

    filas_a_agregar = []

    progress_bar = fc.ProgressBar(
        len(df_importacion), prefix="Procesando filas:", suffix="Completado"
    )

    for index, fila in df_importacion.iterrows():
        clave = str(fila[config.codigo_columna_maestra])

        if clave in claves_maestro:
            logger.info(
                f"Fila {index}: Ignorada - Clave '{clave}' ya existe en el archivo maestro"
            )
            progress_bar.increment()
            continue

        claves_maestro.add(clave)

        fila_filtrada = {
            col: fila[col]
            for col in fila.index
            if col == config.codigo_columna_maestra or fc.es_columna_tipo_xxYY(col)
        }
        filas_a_agregar.append(fila_filtrada)
        progress_bar.increment()

    progress_bar.finish()

    if not filas_a_agregar:
        logger.info("No hay nuevas filas válidas para agregar.")
        print("No hay nuevas filas válidas para agregar.")
        return

    for columna in df_importacion.columns:
        if fc.es_columna_tipo_xxYY(columna) and columna not in df_maestro.columns:
            df_maestro[columna] = None

    df_nuevas_filas = pd.DataFrame(filas_a_agregar)
    df_maestro = pd.concat([df_maestro, df_nuevas_filas], ignore_index=True)

    fc.save_dataframe_with_retries(df_maestro, output_path=archivo_maestro)
    logger.info(
        f"Se han agregado {len(filas_a_agregar)} nuevas filas válidas al archivo maestro."
    )
    print(
        f"Se han agregado {len(filas_a_agregar)} nuevas filas válidas al archivo maestro."
    )


def run(config: TranslationConfig, archivo_importacion):
    global logger
    logger = fc.configurar_logger(config.work_dir)
    script_name = os.path.basename(__file__)
    print(f"\rIniciando: {script_name}\r")
    importar(config, archivo_importacion)


if __name__ == "__main__":
    import menu_pasos_traduccion

    menu_pasos_traduccion.main()
